@@ -37,7 +37,10 @@ struct flash_bsl_device {
 	int		long_password;
 };
 
-#define MAX_PACKET 256
+#define MAX_BLOCK	256
+
+/* This should be at least MAX_BLOCK + 4 */
+#define MAX_PACKET	512
 
 /* adapted from TI's published BSL source code */
 #define CRC_INIT 0xffff
@@ -223,7 +226,7 @@ static int flash_bsl_recv(struct flash_bsl_device *dev,
         debug_hexdump("received message", recv_buf, recv_len);
 #endif
 
-        usleep(10000);
+	delay_ms(10);
 	return recv_len;
 }
 
@@ -263,7 +266,6 @@ static void flash_bsl_perror(uint8_t code) {
 	}
 }
 
-#define MAX_BLOCK 256
 static int flash_bsl_readmem(device_t dev_base,
 			     address_t addr, uint8_t *mem, address_t len)
 {
@@ -548,7 +550,7 @@ static int flash_bsl_writemem(device_t dev_base,
 
 static void entry_delay(void)
 {
-	usleep(1000);
+	delay_ms(1);
 }
 
 static int enter_via_dtr_rts(struct flash_bsl_device *dev)
@@ -655,7 +657,7 @@ static device_t flash_bsl_open(const struct device_args *args)
 	if (enter_via_dtr_rts(dev) < 0)
 		goto fail;
 
-	usleep(500000);
+	delay_ms(500);
 
 	/* unlock device (erase then send password) */
 	if (flash_bsl_unlock(dev) < 0) {
