@@ -26,6 +26,7 @@
 #include "gdb_proto.h"
 #include "opdb.h"
 #include "util.h"
+#include "ctrlc.h"
 
 struct gdb_client {
 	struct device			base;
@@ -216,10 +217,31 @@ static int do_reset(struct gdb_client *dev)
 	return 0;
 }
 
-static int bp_send(struct gdb_data *gdb, int c, address_t addr)
+static int bp_send(struct gdb_data *gdb, int c, address_t addr,
+		   device_bptype_t type)
 {
+	int type_code = 0;
+
+	switch (type) {
+	case DEVICE_BPTYPE_BREAK:
+		type_code = 1;
+		break;
+
+	case DEVICE_BPTYPE_WRITE:
+		type_code = 2;
+		break;
+
+	case DEVICE_BPTYPE_READ:
+		type_code = 3;
+		break;
+
+	case DEVICE_BPTYPE_WATCH:
+		type_code = 4;
+		break;
+	}
+
 	gdb_packet_start(gdb);
-	gdb_printf(gdb, "%c1,%04x,2", c, addr);
+	gdb_printf(gdb, "%c%d,%04x,2", c, type_code, addr);
 	gdb_packet_end(gdb);
 	if (gdb_flush_ack(gdb) < 0)
 		return -1;
@@ -239,11 +261,11 @@ static int refresh_bps(struct gdb_client *dev)
 			continue;
 
 		if ((old->flags & DEVICE_BP_ENABLED) &&
-		    (bp_send(&dev->gdb, 'z', old->addr) < 0))
+		    (bp_send(&dev->gdb, 'z', old->addr, old->type) < 0))
 			return -1;
 
 		if ((bp->flags & DEVICE_BP_ENABLED) &&
-		    (bp_send(&dev->gdb, 'Z', bp->addr) < 0))
+		    (bp_send(&dev->gdb, 'Z', bp->addr, bp->type) < 0))
 			return -1;
 
 		bp->flags &= ~DEVICE_BP_DIRTY;
@@ -327,7 +349,6 @@ static device_status_t gdbc_poll(device_t dev_base)
 	if (!dev->is_running)
 		return DEVICE_STATUS_HALTED;
 
-	ctrlc_reset();
 	len = gdb_peek(&dev->gdb, 50);
 	if (ctrlc_check())
 		return DEVICE_STATUS_INTR;
@@ -381,7 +402,7 @@ static int connect_to(const char *spec)
 	printc_dbg("Looking up %s...\n", hostname);
 	ent = gethostbyname(hostname);
 	if (!ent) {
-#ifdef WIN32
+#ifdef __Windows__
 		printc_err("No such host: %s: %s\n", hostname,
 			   last_error());
 #else
