@@ -77,6 +77,32 @@ static const struct opdb_key keys[] = {
 		.defval = {
 			.numeric = 64
 		}
+	},
+	{
+		.name = "enable_locked_flash_access",
+		.type = OPDB_TYPE_BOOLEAN,
+		.help =
+"If set, some drivers will allow erase/program access to the info A\n"
+"segment. If in doubt, do not enable this.\n",
+		.defval = {
+			.boolean = 0
+		}
+	},
+	{
+		.name = "enable_bsl_access",
+		.type = OPDB_TYPE_BOOLEAN,
+		.help =
+"If set, some drivers will allow erase/program access to flash\n"
+"BSL memory. If in doubt, do not enable this.\n"
+	},
+	{
+		.name = "gdb_default_port",
+		.type = OPDB_TYPE_NUMERIC,
+		.help =
+"Default TCP port for GDB server, if no argument is given.\n",
+		.defval = {
+			.numeric = 2000
+		}
 	}
 };
 
@@ -186,3 +212,15 @@ address_t opdb_get_numeric(const char *name)
 
 	return values[idx].numeric;
 }
+
+fperm_t opdb_read_fperm(void)
+{
+	fperm_t ret = 0;
+
+	if (opdb_get_boolean("enable_locked_flash_access"))
+		ret |= FPERM_LOCKED_FLASH;
+	if (opdb_get_boolean("enable_bsl_access"))
+		ret |= FPERM_BSL;
+
+	return ret;
+}
