@@ -22,111 +22,16 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
-#include <signal.h>
 #include <assert.h>
+#include <time.h>
 
-#ifdef WIN32
+#ifdef __Windows__
 #include <windows.h>
 #endif
 
 #include "util.h"
 #include "output.h"
 
-#ifdef WIN32
-static int ctrlc_flag;
-static HANDLE ctrlc_event;
-static CRITICAL_SECTION ctrlc_cs;
-
-static WINAPI BOOL ctrlc_handler(DWORD event)
-{
-	if (event == CTRL_C_EVENT) {
-		EnterCriticalSection(&ctrlc_cs);
-		ctrlc_flag = 1;
-		LeaveCriticalSection(&ctrlc_cs);
-		SetEvent(ctrlc_event);
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-void ctrlc_init(void)
-{
-	ctrlc_event = CreateEvent(0, TRUE, FALSE, NULL);
-	InitializeCriticalSection(&ctrlc_cs);
-	SetConsoleCtrlHandler(ctrlc_handler, TRUE);
-}
-
-void ctrlc_exit(void)
-{
-	SetConsoleCtrlHandler(NULL, TRUE);
-	DeleteCriticalSection(&ctrlc_cs);
-	CloseHandle(ctrlc_event);
-}
-
-int ctrlc_check(void)
-{
-	int cc;
-
-	EnterCriticalSection(&ctrlc_cs);
-	cc = ctrlc_flag;
-	LeaveCriticalSection(&ctrlc_cs);
-
-	return cc;
-}
-
-void ctrlc_reset(void)
-{
-	EnterCriticalSection(&ctrlc_cs);
-	ctrlc_flag = 0;
-	LeaveCriticalSection(&ctrlc_cs);
-	ResetEvent(ctrlc_event);
-}
-
-HANDLE ctrlc_win32_event(void)
-{
-	return ctrlc_event;
-}
-#else /* WIN32 */
-static volatile int ctrlc_flag;
-
-static void sigint_handler(int signum)
-{
-	(void)signum;
-
-	ctrlc_flag = 1;
-}
-
-void ctrlc_init(void)
-{
-#if defined(__CYGWIN__)
-       signal(SIGINT, sigint_handler);
-#else
-       static const struct sigaction siga = {
-               .sa_handler = sigint_handler,
-               .sa_flags = 0
-       };
-
-       sigaction(SIGINT, &siga, NULL);
-#endif
-}
-
-void ctrlc_exit(void)
-{
-	signal(SIGINT, SIG_DFL);
-}
-
-void ctrlc_reset(void)
-{
-	ctrlc_flag = 0;
-}
-
-int ctrlc_check(void)
-{
-	return ctrlc_flag;
-}
-#endif
-
 char *get_arg(char **text)
 {
 	char *start;
@@ -262,7 +167,7 @@ int hexval(int c)
 	return 0;
 }
 
-#ifdef WIN32
+#ifdef __Windows__
 char *strsep(char **strp, const char *delim)
 {
 	char *start = *strp;
@@ -292,7 +197,7 @@ char *strsep(char **strp, const char *delim)
 }
 #endif
 
-#ifdef WIN32
+#ifdef __Windows__
 const char *last_error(void)
 {
 	DWORD err = GetLastError();
@@ -360,3 +265,72 @@ char *expand_tilde(const char *path)
 	/* Caller must free()! */
 	return expanded;
 }
+
+#ifdef __Windows__
+int delay_s(unsigned int s)
+{
+	Sleep(s * 1000);
+
+	return 0;
+}
+
+int delay_ms(unsigned int s)
+{
+	Sleep(s);
+
+	return 0;
+}
+#else
+int delay_s(unsigned int s)
+{
+	return sleep(s);
+}
+
+int delay_ms(unsigned int s)
+{
+	struct timespec ts;
+
+	ts.tv_sec = s / 1000;
+	ts.tv_nsec = (s % 1000) * 1000000;
+
+	return nanosleep(&ts, NULL);
+}
+#endif
+
+int base64_encode(const uint8_t *src, int len, char *dst, int max_len)
+{
+	static const char basis[] =
+	    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+	    "abcdefghijklmnopqrstuvwxyz0123456789+/";
+	int i = 0;
+	int k = 0;
+
+	while ((i < len) && (k + 4) < max_len) {
+		int a = src[i++];
+
+		dst[k++] = basis[a >> 2];
+
+		if (i < len) {
+			int b = src[i++];
+
+			dst[k++] = basis[((a & 3) << 4) | (b & 0xf0) >> 4];
+			if (i < len) {
+				int c = src[i++];
+
+				dst[k++] = basis[((b & 0xf) << 2) |
+						 ((c & 0xc0) >> 6)];
+				dst[k++] = basis[c & 0x3f];
+			} else {
+				dst[k++] = basis[(b & 0xf) << 2];
+				dst[k++] = '=';
+			}
+		} else {
+			dst[k++] = basis[(a & 3) << 4];
+			dst[k++] = '=';
+			dst[k++] = '=';
+		}
+	}
+
+	dst[k] = 0;
+	return i;
+}
